/*
 * robotfindskitten-gba - A portable Zen simulation
 * Copyright (C) 2001,2002 Benjamin Molitor
 *                    b.molitor@gmx.net
 *
 * Original robotfindskitten is
 * Copyright (C) 1997,2000 Leonard Richardson 
 *                         leonardr@segfault.org
 * 
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or EXISTANCE OF KITTEN.  See the GNU General
 *   Public License for more details.
 *
 *   http://www.gnu.org/copyleft/gpl.html
 *
 */

#include <string.h>
#include <stdlib.h>
#include "gba.h"
#include "keypad.h"
#include "time.h"
#include "text.h"
#include "messages.h"

#define EMPTY -1
#define ROBOT 0
#define KITTEN 1
#define NUM_BOGUS 10

struct object
{
	int x;
	int y;
	int color;
	char character;
};

object robot;
object kitten;
object empty;
object bogus[MESSAGES];
int bogus_messages[MESSAGES];
int used_messages[MESSAGES];
int screen[30][20];


void clrmsgs()	/* clear messages area */
{
	int x=0, y=16;
	for(x=0; x<7200; x++)
	{
		VideoBuffer[x] = RGB(0,0,16);
	}

	/* draw blue gradient thingy */
	for(x=7200; x<7680; x++)
	{
		if (x%240 == 0) y = y-6;
		VideoBuffer[x] = RGB(0,0,y);
	}
}


void meet(void)
{
	clrscr(RGB(0,0,0));
	int x=0, y=16, temp=0;
	for (x=0; x < 96; x++)
	{

		plotletter(20+(x-1), 48, 26, RGB(0,0,0), RGB(0,0,0));		
		plotletter(20+x, 48, robot.character, robot.color, RGB(0,0,0));
		plotletter(220-(x-1), 48, 26, RGB(0,0,0), RGB(0,0,0));
		plotletter(220-x, 48, kitten.character, kitten.color, RGB(0,0,0));
		for (temp=0; temp<1000; temp++){}
	}
	gprint(6, 4, "You found kitten!", RGB(31,31,31), RGB(0,0,0));
	gprint(6, 8, "Way to go, robot!", RGB(31,31,31), RGB(0,0,0));
	gprint(10, 19, "Press Start.", RGB(31,31,31), RGB(0,0,0));

}


void process_input(int input)
{
	char bla[200];		/* temp variable for message wrapping */
	int check_x = robot.x;
	int check_y = robot.y;
	switch (input)
	{
		case 1: /* up */
			check_y--;
			break;
		case 2: /* down */
			check_y++;
			break;
		case 3: /* left */
			check_x--;
			break;
		case 4: /* right */
			check_x++;
			break;
		case 0:
			break;
		default: /* invalid command */
			return;
	}

	/* Check for going off the edge of the screen. */
	if (check_y < 4 || check_y > 19 || check_x < 0 || check_x > 29)
	{
		return;
	}

	/* Check for collision */
	if (screen[check_x][check_y] != EMPTY)
	{
		switch (screen[check_x][check_y])
		{
			case ROBOT:
				/* We didn't move. */
				break;
			case KITTEN: /* Found it! */
				clrmsgs();
				meet();
				while(GetInput()!=5){}
				asm ("swi 0"); /* soft reset */
				break;
			default:
				clrmsgs();
				strcpy(bla, messages[bogus_messages[screen[check_x][check_y]-2]]);
				gprint(0, 0, wrap(bla, 30), RGB(31,31,31), RGB(0,0,16));
				break;
		}
		return;
	}
	/* Otherwise, move the robot. */
	robot.x = check_x;
	robot.y = check_y;
}


void drawmenu(void)
{

	clrscr(RGB(0,0,0));
	/* pretty pictures! */
	gprint(7, 6, "robotfindskitten", RGB(31,31,31), RGB(0,0,0));
	gprint(3, 1, "[-]", RGB(20,20,20), RGB(0,0,0));
	gprint(3, 2, "(+)=C", RGB(20,20,20), RGB(0,0,0));
	gprint(3, 3, "| |", RGB(20,20,20), RGB(0,0,0));
	gprint(3, 4, "OOO", RGB(20,20,20), RGB(0,0,0));
	gprint(20, 1, "|\\_/|", RGB(20,20,20), RGB(0,0,0));
	gprint(20, 2, "|o o|__", RGB(20,20,20), RGB(0,0,0));
	gprint(20, 3, "==*==__\\", RGB(20,20,20), RGB(0,0,0));
	gprint(20, 4, "C_C_(___)", RGB(20,20,20), RGB(0,0,0));

	gprint(9, 10, "Find Kitten!", RGB(27,27,31), RGB(0,0,0));
	gprint(9, 11, "Instructions", RGB(27,27,31), RGB(0,0,0));
	gprint(9, 12, "About", RGB(27,27,31), RGB(0,0,0));

	gprint(14, 19, "V. 0.2.5.349-GBA", RGB(20,20,20), RGB(0,0,0));	/* Now THAT's a version number! */
									/* Would be nice if this was automated... */
}


void instructions(void)
{
	clrscr(RGB(0,0,0));

	gprint(7, 0, "robotfindskitten", RGB(31,31,31), RGB(0,0,0));
	gprint(1, 6, "In this game, you are robot", RGB(31,31,31), RGB(0,0,0));
	gprint(1, 7, "(#). Your job is to find", RGB(31,31,31), RGB(0,0,0));
	gprint(2, 7, "#", doscolor(7), RGB(0,0,0));
	gprint(1, 8, "kitten. This task is", RGB(31,31,31), RGB(0,0,0));
	gprint(1, 9, "complicated by the existance", RGB(31,31,31), RGB(0,0,0));
	gprint(1, 10, "of various things which are", RGB(31,31,31), RGB(0,0,0));
	gprint(1, 11, "not kitten.", RGB(31,31,31), RGB(0,0,0));
	gprint(1, 12, "Robot must touch items to", RGB(31,31,31), RGB(0,0,0));
	gprint(1, 13, "determine if they are kitten", RGB(31,31,31), RGB(0,0,0));
	gprint(1, 14, "or not. The game ends when", RGB(31,31,31), RGB(0,0,0));
	gprint(1, 15, "robotfindskitten.", RGB(31,31,31), RGB(0,0,0));
	while(!GetInput()){}
	drawmenu();
}


void about(void)
{
	clrscr(RGB(0,0,0));
	gprint(7, 0, "robotfindskitten", RGB(31,31,31), RGB(0,0,0));
	gprint(3, 2, "1997, 2000 L. Richardson", RGB(16,16,16), RGB(0,0,0));
	gprint(3, 5, "Gameboy Advance adaption", RGB(23,23,23), RGB(0,0,0));
	gprint(4, 7, "2002 Benjamin Molitor", RGB(16,16,16), RGB(0,0,0));
	gprint(6, 8, "b.molitor@gmx.net", RGB(16,16,16), RGB(0,0,0));
	gprint(2, 12, "Written originally for the", RGB(31,31,31), RGB(0,0,0));
	gprint(2, 13, "Nerth Pork robotfindskitten", RGB(31,31,31), RGB(0,0,0));
	gprint(2, 14, "contest.", RGB(31,31,31), RGB(0,0,0));
	gprint(3, 18, "www.robotfindskitten.org", RGB(23,23,23), RGB(0,0,0));
	while(!GetInput()){}
	drawmenu();
}


void menu(void)
{
	int selection=0, temp=0;
	drawmenu();
	while(1)
	{
		gprint(8, 10+selection, ">", RGB(31,0,0), RGB(0,0,0));	/* Print leetle arrow */
		for (temp=0; temp<30000; temp++){};	/* CPU is waaay too fast for my fingers! */
		switch(GetInput())
		{

			case 1: /* UP */
				gprint(8, 10+selection, " ", RGB(0,0,0), RGB(0,0,0));		
				selection--;
				if (selection == -1) selection=2;
				break;
			case 2: /* DOWN */
				gprint(8, 10+selection, " ", RGB(0,0,0), RGB(0,0,0));		
				selection++;
				if (selection == 3) selection=0;
				break;
			case 6: /* A */
				if (selection == 0) return;
				if (selection == 1) instructions();
				if (selection == 2) about();
				break;
		}
	}

}


int main(void)
{

	int counter, counter2, dummy;

	u16* ScreenBuffer = (u16*)0x6000000;	/* this is the start of video memory */

	SetMode(MODE_3 | BG2_ENABLE);	/* this sets the screen mode to mode three and enables background 2 */

	REG_TM0CNT = TIME_FREQUENCY_1024 | TIME_ENABLE;	/* enable timer, used to initialize random number generator later */

	menu();
	clrscr(RGB(0,0,0));
	clrmsgs();

	srand(REG_TM0D);	/* use the time we waited at the startup screen as random seed... better ideas, anyone? */

	/* calculate random positions etc for robot, kitten and nki */
	empty.x = -1;
	empty.y = -1;
	empty.color = 0;
	empty.character = ' ';
	
	for (counter = 0; counter < 30; counter++)
	{
		for (counter2 = 0; counter2 < 20; counter2++)
		{
			screen[counter][counter2] = EMPTY;
		}
	}


	for (counter = 0; counter < MESSAGES; counter++)
	{
		used_messages[counter] = 0;
		bogus_messages[counter] = 0;
		bogus[counter] = empty;
	}

	/* initialize robot */
	robot.x = rand() % 29;
	robot.y = rand() % 15+5;
	robot.character = '#';
	robot.color = RGB(20,20,20);
	screen[robot.x][robot.y] = ROBOT;

	/* initialize kitten */
	do
	{
		kitten.x = rand() % 29;
		kitten.y = rand() % 15+5;
	} while (screen[kitten.x][kitten.y] != EMPTY);

	do
	{
		kitten.character = rand() % 95 + 32;
	} while (kitten.character == '#' || kitten.character == ' ' || kitten.character == char(7));
	kitten.color = doscolor(rand() % 14 + 1);
	screen[kitten.x][kitten.y] = KITTEN;


	/* initialize bogus objects */
	for (counter = 0; counter < NUM_BOGUS; counter++)
	{
		/* Assign a unique position. */
		do
		{
			bogus[counter].x = rand() % 29;
			bogus[counter].y = rand() % 15 + 5;
		} while (screen[bogus[counter].x][bogus[counter].y] != EMPTY);
		screen[bogus[counter].x][bogus[counter].y] = counter+2;

		/* Assign a character. */
		do
		{
			bogus[counter].character = rand() % 95 + 32;
		} while (bogus[counter].character == '#' || bogus[counter].character == ' ' || bogus[counter].character == char(7));
		bogus[counter].color = doscolor(rand() % 14 + 1);

		/* Assign a unique message. */
		int index = 0;
		do
		{
			index = rand() % MESSAGES;
		} while (used_messages[index] != 0);
		bogus_messages[counter] = index;
		used_messages[index] = 1;
	}

	plotletter(kitten.x*8, kitten.y*8, kitten.character, kitten.color, RGB(0,0,0));	

	for (counter = 0; counter < NUM_BOGUS; counter++)
	{
		plotletter(bogus[counter].x*8, bogus[counter].y*8, bogus[counter].character, bogus[counter].color, doscolor(0));
	}


	plotletter(robot.x*8, robot.y*8, robot.character, robot.color, doscolor(0));	
	int old_x = robot.x;
	int old_y = robot.y;

	while (1)
	{
		for(dummy=0; dummy < 20000; dummy++){};	/* It's the all-famous way-too-fast-loop again! */
		process_input(GetInput());
		/* Redraw robot, where avaliable */
		if (!(old_x == robot.x && old_y == robot.y))
		{
			plotletter(old_x*8, old_y*8, 32, RGB(0,0,0), RGB(0,0,0));
			plotletter(robot.x*8, robot.y*8, robot.character, robot.color, RGB(0,0,0));
			screen[old_x][old_y] = EMPTY;
			screen[robot.x][robot.y] = ROBOT;
			old_x = robot.x;
			old_y = robot.y;
		}
	}
}
