/* time.h by staringmonkey */

#ifndef TIME_H
#define TIME_H

#define TIME_FREQUENCY_SYSTEM 0x0
#define TIME_FREQUENCY_64 0x1
#define TIME_FREQUENCY_256 0x2
#define TIME_FREQUENCY_1024 0x3
#define TIME_OVERFLOW 0x4
#define TIME_ENABLE 0x80
#define TIME_IRQ_ENABLE 0x40

void WaitTime(int seconds, int milliSeconds)
{
	/* Enable timers */
	REG_TM2CNT = TIME_FREQUENCY_256 | TIME_ENABLE;
	REG_TM3CNT = TIME_OVERFLOW | TIME_ENABLE;

	/* Wait until ? seconds have passed */
	while(REG_TM3D < seconds) { }

	/* Zero out base/millisecond timer */
	REG_TM2D = 0;

	/* 1 millisecond = max register value (65536) divided by the # milliseconds in a second (1000)
	   Wait until ? milliseconds have passed */
	while(REG_TM2D * 65536/1000 < milliSeconds) { }

	/* Disable timers */
	REG_TM2CNT = 0;
	REG_TM3CNT = 0;
	
	/* Zero out timer values */
	REG_TM2D = 0;
	REG_TM3D = 0;
}

void waitforvsync(void)
{
	volatile u16* ScanlineCounter = (volatile u16*)0x4000006;
        while(*ScanlineCounter < 160) { }
}

#endif
